@@ -17,6 +17,9 @@ class EventListener(ABC):
     def on_connected(self):
         pass
 
+    def on_disconnected(self):
+        pass
+
     def on_keep_alive_event(self, event):
         pass
 
@@ -50,7 +53,7 @@ class Device(EventListener):
 
     def _perform_get(self, path:str) -> Dict[str, Any]:
         uri = self._uri + path
-        logging.info("query GET " + uri)
+        #logging.info("query GET " + uri)
         response = requests.get(uri, headers={"Authorization": "Bearer " + self._auth.access_token})
         if self.__is_success(response.status_code):
             return response.json()
@@ -61,7 +64,7 @@ class Device(EventListener):
 
     def _perform_put(self, path:str, data: str, max_trials: int = 3, current_trial: int = 1):
         uri = self._uri + path
-        logging.info("query PUT " + uri + " (" + str(current_trial) + " trial)")
+        #logging.info("query PUT " + uri + " (" + str(current_trial) + " trial)")
         response = requests.put(uri, data=data, headers={"Content-Type": "application/json", "Authorization": "Bearer " + self._auth.access_token})
         if not self.__is_success(response.status_code):
             logging.warning("error occurred by calling PUT " + uri + " " + data)
@@ -130,8 +133,7 @@ class Dishwasher(Device):
         self.__refresh()
 
     def on_keep_alive_event(self, event):
-        for value_changed_listener in self._value_changed_listeners:
-            value_changed_listener()
+        logging.debug("keep alive event")
 
     def on_notify_event(self, event):
         logging.debug("notify event: " + str(event.data))
@@ -148,25 +150,25 @@ class Dishwasher(Device):
         if event.id == self.haid:
             try:
                 data = json.loads(event.data)
-                self.__on_value_changes(data.get('items', []))
+                self.__on_value_changes(data.get('items', []), "updated")
                 self.__notify_listeners()
             except Exception as e:
                 logging.warning("error occurred by handling event " + str(event), e)
 
-    def __on_value_changes(self, changes: List[Any]):
+    def __on_value_changes(self, changes: List[Any], ops: str = "updated"):
         for record in changes:
             if record['key'] == 'BSH.Common.Status.DoorState':
                 self.__door = record['value']
-                logging.info("door state updated: " + str(self.__door))
+                logging.info("door state " + ops + ": " + str(self.__door))
             elif record['key'] == 'BSH.Common.Status.OperationState':
                 self.__operation = record['value']
-                logging.info("operation state updated: " + str(self.__operation))
+                logging.info("operation state " + ops + ": " + str(self.__operation))
             elif record['key'] == 'BSH.Common.Status.RemoteControlStartAllowed':
                 self.remote_start_allowed = record['value']
-                logging.info("remote start allowed updated: " + str(self.remote_start_allowed))
+                logging.info("remote start allowed " + ops + ": " + str(self.remote_start_allowed))
             elif record['key'] == 'BSH.Common.Setting.PowerState':
                 self.__power = record['value']
-                logging.info("power state updated: " + str(self.__power))
+                logging.info("power state " + ops + ": " + str(self.__power))
             elif record['key'] == 'BSH.Common.Root.SelectedProgram':
                 self.__program_selected = record['value']
             elif record['key'] ==  'BSH.Common.Root.ActiveProgram':
@@ -188,14 +190,16 @@ class Dishwasher(Device):
             else:
                 logging.info("unknown changed " + str(record))
 
-    def __refresh(self):
+    def __refresh(self, notify: bool = True):
         try:
-            self.__on_value_changes(self._perform_get('/settings')['data']['settings'])
-            self.__on_value_changes(self._perform_get('/status')['data']['status'])
+            logging.info("fetch settings, status and selection")
+            self.__on_value_changes(self._perform_get('/settings')['data']['settings'], "fetched")
+            self.__on_value_changes(self._perform_get('/status')['data']['status'], "fetched")
             record = self._perform_get('/programs/selected')['data']
             self.__program_selected = record['key']
-            self.__on_value_changes(record['options'])
-            self.__notify_listeners()
+            self.__on_value_changes(record['options'], "fetched")
+            if notify:
+                self.__notify_listeners()
         except Exception as e:
             logging.warning("error occurred on refreshing", e)
 
@@ -231,7 +235,7 @@ class Dishwasher(Device):
             return ""
 
     def set_start_date(self, dt: str):
-        self.__refresh()
+        self.__refresh(notify=False)
         if self.__operation == "BSH.Common.EnumType.OperationState.Ready":
             remaining_secs_to_wait = int((datetime.fromisoformat(dt) - datetime.now()).total_seconds())
             if remaining_secs_to_wait < 0:
@@ -244,11 +248,11 @@ class Dishwasher(Device):
             data = {
                 "data": {
                     "key": self.__program_selected,
-                    "options": [ {
+                    "options": [{
                                     "key": "BSH.Common.Option.StartInRelative",
                                     "value": remaining_secs_to_wait,
                                     "unit": "seconds"
-                                } ]
+                                }]
                 }
             }
             try:
@@ -257,7 +261,8 @@ class Dishwasher(Device):
             except Exception as e:
                 logging.warning("error occurred by starting dishwasher", e)
         else:
-            logging.info("ignore starting. Dishwasher is " + self.__operation + "state")
+            logging.info("ignoring start command. Dishwasher is in state " + self.__operation)
+        self.__refresh()
 
     def __str__(self):
         return "power=" + str(self.power) + \
@@ -292,20 +297,23 @@ class HomeConnect:
         num_reconnects = 0
         while True:
             try:
-                self.__consume_sse_events(uri, max_connection_time_minutes=55)
+                self.__consume_sse_events(uri, read_timeout_sec=3 * 60, max_lifetime_sec=45 * 60)
                 num_reconnects = 0
             except Exception as e:
-                logging.warning("Event stream (" + uri + ") error: " + str(e))
+                logging.warning("Event stream (" + uri + ") error: ", e)
                 wait_time_sec = {0: 3, 1:5, 2: 30, 3: 2*60, 4: 5*60}.get(num_reconnects, 20*60)
                 num_reconnects += 1
                 logging.info("try " + str(num_reconnects) + ". reconnect in " + str(wait_time_sec) + " sec")
                 sleep(wait_time_sec)
 
-    def __consume_sse_events(self, uri: str, max_connection_time_minutes: int):
+    def __consume_sse_events(self, uri: str, read_timeout_sec: int, max_lifetime_sec:int):
         client = None
         try:
-            logging.info("opening event stream connection " + uri)
-            response = requests.get(uri, stream=True, headers={'Accept': 'text/event-stream', "Authorization": "Bearer " + self.auth.access_token})
+            logging.info("opening event stream connection " + uri + "(read timeout " + str(read_timeout_sec) + " sec, lifetimeout " + str(max_lifetime_sec) + " sec)")
+            response = requests.get(uri,
+                                    stream=True,
+                                    timeout=read_timeout_sec,
+                                    headers={'Accept': 'text/event-stream', "Authorization": "Bearer " + self.auth.access_token})
             if response.status_code == 200:
                 client = sseclient.SSEClient(response)
                 logging.info("consuming events...")
@@ -329,16 +337,18 @@ class HomeConnect:
                             notify_listener.on_event_event(event)
                     else:
                         logging.info("unknown event type " + str(event.event))
-                    # max connection time reached?
-                    if datetime.now() > (connect_time + timedelta(minutes=max_connection_time_minutes)):
-                        logging.info("closing event stream. Max connect time " + str(max_connection_time_minutes) + " min reached (periodic reconnect)")
+                    if datetime.now() > (connect_time + timedelta(seconds=max_lifetime_sec)):
+                        logging.info("closing event stream. Max lifetime " + str(max_lifetime_sec) + " sec reached (periodic reconnect)")
                         return
             else:
                 logging.warning("error occurred by opening event stream connection " + uri)
                 logging.warning("got " + str(response.status_code) + " " + response.text)
         finally:
+            for notify_listener in self.notify_listeners:
+                notify_listener.on_disconnected()
+            logging.info("event stream closed")
             if client is not None:
-                client.close()
+                    client.close()
 
     def devices(self) -> List[Device]:
         uri = HomeConnect.API_URI + "/homeappliances"
